package com.migao.service.impl;

import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.response.ResponseBuilder;
import com.migao.entity.po.Project;
import com.migao.entity.vo.req.ProjectInsertReq;
import com.migao.entity.vo.req.ProjectPageQueryReq;
import com.migao.entity.vo.req.ProjectQueryReq;
import com.migao.entity.vo.req.ProjectUpdateReq;
import com.migao.entity.vo.res.ProjectQueryRes;
import com.migao.mapper.ProjectMapper;
import com.migao.mapper.ProjectRepository;
import com.migao.service.ProjectService;
import com.migao.util.EntityUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.time.LocalDateTime;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = {Exception.class})
public class ProjectServiceImpl implements ProjectService {

    @Resource
    private ProjectRepository projectRepository;

    @Resource
    private  ProjectMapper projectMapper;


    /**
     * 增
     *
     * @param projectInsertReq projectInsertReq
     * @return ?
     */
    @Override
    public ResponseBean<?> insert(ProjectInsertReq projectInsertReq) {
        return ResponseBuilder.ok(projectRepository.save(EntityUtils.copyProperties(projectInsertReq, Project.class)));
    }

    /**
     * 分页查
     *
     * @param projectPageQueryReq projectPageQueryReq
     * @return ?
     */
    @Override
    public ResponseBean<PageBean<ProjectQueryRes>> pageQuery(ProjectPageQueryReq projectPageQueryReq) {
        //1。分页查询所有供公司
        Integer page = projectPageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = projectPageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;
        PageRequest pageRequest = PageRequest.of(page - 1, size);
        Specification<Project> projectSpecification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();

                    if (projectPageQueryReq.getCreateTime()!=null) {
                    Predicate equal = criteriaBuilder.equal(root.get("createTime").as(LocalDateTime.class),
                    projectPageQueryReq.getCreateTime()
                    );
                    predicateList.add(equal);
                    }
                    if (StringUtils.isNotBlank(projectPageQueryReq.getName())) {
                        Predicate like = criteriaBuilder.like(root.get("name").as(String.class),
                        StringUtils.join(
                        "%", projectPageQueryReq.getName(), "%"
                        ));
                        predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectPageQueryReq.getOrderStatus())) {
                        Predicate like = criteriaBuilder.like(root.get("orderStatus").as(String.class),
                        StringUtils.join(
                        "%", projectPageQueryReq.getOrderStatus(), "%"
                        ));
                        predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectPageQueryReq.getReserveStatus())) {
                        Predicate like = criteriaBuilder.like(root.get("reserveStatus").as(String.class),
                        StringUtils.join(
                        "%", projectPageQueryReq.getReserveStatus(), "%"
                        ));
                        predicateList.add(like);
                    }
            criteriaBuilder.equal(root.get("deleted").as(Integer.class),1);
            Predicate[] predicateArray = new Predicate[predicateList.size()];
            criteriaQuery.where(criteriaBuilder.and(predicateList.toArray(predicateArray)));
            return criteriaQuery.getRestriction();
        };
        Page<Project> all = projectRepository.findAll(projectSpecification, pageRequest);
        return ResponseBuilder.ok(
                all,
                element -> {
                    ProjectQueryRes projectQueryRes = ProjectQueryRes
                            .builder()
                            .id(element.getId())
                            .capital(String.valueOf(element.getCapital()))
                            .content(element.getContent())
                            .cost(String.valueOf(element.getCost()))
                            .createTime(element.getCreateTime())
                            .createUserId(element.getCreateUserId())
                            .deleted(element.getDeleted())
                            .level(element.getLevel())
                            .name(element.getName())
                            .progress(element.getProgress())
                            .projectClassify(element.getProjectClassify())
                            .specialtyClassify(element.getSpecialtyClassify())
                            .total(String.valueOf(element.getTotal()))
                            .totalInvestment(String.valueOf(element.getTotalInvestment()))
                            .twoClassify(element.getTwoClassify())
                            .unit(element.getUnit())
                            .orderStatus(String.valueOf(element.getOrderStatus()))
                            .reserveStatus(String.valueOf(element.getReserveStatus()))
                            .build();
                    return projectQueryRes;
                }
        );
    }

    /**
    * 删
    *
    * @param id id
    * @return ?
    */
    @Override
    public ResponseBean<?> deleteById(Integer id) {
        projectRepository.findById(id).ifPresent(
            i -> {
                i.setDeleted(1);
        projectRepository.save(i);
            }
        );
        return ResponseBuilder.ok();
    }

    /**
    * 批量删
    *
    * @param ids ids
    * @return ?
    */
    @Override
    public ResponseBean<?> deleteBatch(List<Integer> ids) {
            projectRepository.findAllById(ids).forEach(
            i -> {
                i.setDeleted(1);
            projectRepository.save(i);
            }
        );
        return ResponseBuilder.ok();
    }
    /**
     * 改
     *
     * @param projectUpdateReq projectUpdateReq
     * @return ?
     */
    @Override
    public ResponseBean<?> update(ProjectUpdateReq projectUpdateReq) {
            projectRepository.save(EntityUtils.copyProperties(projectUpdateReq, Project.class));
        return ResponseBuilder.ok();
    }

    /**
     * 查
     *
     * @param projectQueryReq projectQueryReq
     * @return ?
     */
    @Override
    public ResponseBean<List<ProjectQueryRes>> query(ProjectQueryReq projectQueryReq) {
        //1。分页查询所有供公司
        Specification<Project> projectSpecification = (root, criteriaQuery, criteriaBuilder) -> {
        List<Predicate> predicateList = new ArrayList<>();
                    if (projectQueryReq.getId()!=null) {
                    Predicate equal = criteriaBuilder.equal(root.get("id").as(Integer.class),
                    projectQueryReq.getId()
                    );
                    predicateList.add(equal);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getCapital())) {
                    Predicate like = criteriaBuilder.like(root.get("capital").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getCapital(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getContent())) {
                    Predicate like = criteriaBuilder.like(root.get("content").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getContent(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getCost())) {
                    Predicate like = criteriaBuilder.like(root.get("cost").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getCost(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (projectQueryReq.getCreateTime()!=null) {
                    Predicate equal = criteriaBuilder.equal(root.get("createTime").as(LocalDateTime.class),
                    projectQueryReq.getCreateTime()
                    );
                    predicateList.add(equal);
                    }
                    if (projectQueryReq.getCreateUserId()!=null) {
                    Predicate equal = criteriaBuilder.equal(root.get("createUserId").as(Integer.class),
                    projectQueryReq.getCreateUserId()
                    );
                    predicateList.add(equal);
                    }
                    if (projectQueryReq.getDeleted()!=null) {
                    Predicate equal = criteriaBuilder.equal(root.get("deleted").as(Integer.class),
                    projectQueryReq.getDeleted()
                    );
                    predicateList.add(equal);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getLevel())) {
                    Predicate like = criteriaBuilder.like(root.get("level").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getLevel(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getName())) {
                    Predicate like = criteriaBuilder.like(root.get("name").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getName(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getProgress())) {
                    Predicate like = criteriaBuilder.like(root.get("progress").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getProgress(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getProjectClassify())) {
                    Predicate like = criteriaBuilder.like(root.get("projectClassify").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getProjectClassify(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getSpecialtyClassify())) {
                    Predicate like = criteriaBuilder.like(root.get("specialtyClassify").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getSpecialtyClassify(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getTotal())) {
                    Predicate like = criteriaBuilder.like(root.get("total").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getTotal(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getTotalInvestment())) {
                    Predicate like = criteriaBuilder.like(root.get("totalInvestment").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getTotalInvestment(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getTwoClassify())) {
                    Predicate like = criteriaBuilder.like(root.get("twoClassify").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getTwoClassify(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getUnit())) {
                    Predicate like = criteriaBuilder.like(root.get("unit").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getUnit(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getOrderStatus())) {
                    Predicate like = criteriaBuilder.like(root.get("orderStatus").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getOrderStatus(), "%"
                    ));
                    predicateList.add(like);
                    }
                    if (StringUtils.isNotBlank(projectQueryReq.getReserveStatus())) {
                    Predicate like = criteriaBuilder.like(root.get("reserveStatus").as(String.class),
                    StringUtils.join(
                    "%", projectQueryReq.getReserveStatus(), "%"
                    ));
                    predicateList.add(like);
                    }
            criteriaBuilder.equal(root.get("deleted").as(Integer.class),1);
            Predicate[] predicateArray = new Predicate[predicateList.size()];
            criteriaQuery.where(criteriaBuilder.and(predicateList.toArray(predicateArray)));
            return criteriaQuery.getRestriction();
        };
        List<Project> all = projectRepository.findAll(projectSpecification);
        return ResponseBuilder.ok(
                all.stream().map(
                        element -> {
                        ProjectQueryRes projectQueryRes = ProjectQueryRes
                        .builder()
                            .id(element.getId())
                            .capital(String.valueOf(element.getCapital()))
                            .content(element.getContent())
                            .cost(String.valueOf(element.getCost()))
                            .createTime(element.getCreateTime())
                            .createUserId(element.getCreateUserId())
                            .deleted(element.getDeleted())
                            .level(element.getLevel())
                            .name(element.getName())
                            .progress(element.getProgress())
                            .projectClassify(element.getProjectClassify())
                            .specialtyClassify(element.getSpecialtyClassify())
                            .total(String.valueOf(element.getTotal()))
                            .totalInvestment(String.valueOf(element.getTotalInvestment()))
                            .twoClassify(element.getTwoClassify())
                            .unit(element.getUnit())
                            .orderStatus(String.valueOf(element.getOrderStatus()))
                            .reserveStatus(String.valueOf(element.getReserveStatus()))
                        .build();
                        return projectQueryRes;
                        }
                ).collect(Collectors.toList())
        );
    }
}
